@@ -34,7 +34,7 @@ pub use collections::Mpmc;
 ///
 /// Implementors should be simple wrappers around `u32` that allow application code to clearly
 /// differentiate the relevant event sources. Events and signals correspond to the bits of the
-// value returned by `as_bits()`.
+/// value returned by `as_bits()`.
 ///
 /// # Examples
 ///
@@ -89,10 +89,12 @@ pub struct Events<S> {
 /// The type parameter `S` is an [`EventMask`].
 ///
 /// # Delayed signals
-/// The executor will examine the event mask after polling the future to check for any immediate
-/// updates. Since the raised signal set remains frozen during polling, any signal raised by the
-/// future itself won't become visible until this happens. This also means that external events are
-/// not tested for until the future yields. Correct programs are not able to observe this behavior.
+/// The executor will examine the event mask after polling the future to check for any
+/// immediate updates. Since the raised signal set remains frozen during polling, any signal
+/// raised by the future itself through [`Events`] won't become visible until this happens.
+/// This also means that external events are not tested for until the future yields. As an
+/// optimization for the first case, [`Signals::raise()`] updates the raised signal set
+/// immediately. Correct programs are not able to observe this behavior.
 pub struct Signals<'a, S> {
     pending: &'a Events<S>,
     run: Cell<Run>,
@@ -337,6 +339,23 @@ impl<'a, S: EventMask> Signals<'a, S> {
         }
     }
 
+    /// Raise a signal set without delays.
+    ///
+    /// This will perform the equivalent of [`pending().raise(signals)`], but it will
+    /// also update the raised signal set. This prevents the "delayed signals" effect
+    /// that is shown above. Prefer `raise()` if another branch of this future (such as
+    /// in [`futures::join!()`] might benefit from this, otherwise use `pending().raise()`.
+    /// It is impossible to visibly influence a well-behaved poll function by switching
+    /// between `raise()` or `pending().raise()` at any call site; this is a best-effort
+    /// optimization only.
+    pub fn raise(&self, signals: S) {
+        self.pending.raise(signals);
+
+        let run = self.run.get();
+        let raised = run.raised | signals.as_bits();
+        self.run.set(Run { raised, ..run });
+    }
+
     /// Retrieve the underlying event mask.
     pub fn pending(&self) -> &Events<S> {
         self.pending
@@ -389,8 +408,8 @@ impl<'a, S: EventMask> Signals<'a, S> {
     /// This is a variant of [`Signals::drive()`] intended for cases where there is no
     /// proper error type. Although `drive(sig, poll).await.unwrap()` works the same, it
     /// often requires explicit type annotations if `poll` is a closure. This method should
-    /// be preferre in such cases, as well as when `poll` is a wrapper around
-    /// `option.ok_or(WouldBlock)`.
+    /// be preferred in such cases, such as when `poll` is just a wrapper around
+    /// `some_option.ok_or(WouldBlock)`.
     pub async fn drive_infallible<T, F>(&self, signals: S, poll: F) -> T
     where
         F: FnMut() -> nb::Result<T, Infallible>,
@@ -550,7 +569,7 @@ where
 }
 
 impl<'a> Park<'a> {
-    /// Promise that new events won't race with sleeps and get a proof of parking.
+    /// Promise that new events won't race with sleeps, then get a proof of parking.
     ///
     /// Park functions must call this method to obtain the [`Parked`] object that they
     /// return, which also allows them to determine sleep permissibility. The caller
